//
//  AWTableController.m
//  AWTableView
//
//  Created by Ikuo Kudo on 12/07/20.
//  Copyright (c) 2012年  All rights reserved.
//

#import "AWTableController.h"
#import "AWTableViewController.h"
#import "AWTableCellContainer.h"

@interface AWTableController ()


@property ( nonatomic, strong ) NSMutableDictionary* stockedCells;
- ( int ) rowOfVisibleAtStageIndex: ( int ) iStage;

@end


@implementation AWTableController
@synthesize stockedCells;
@synthesize controller;
@synthesize rowMax, columnMax;
@synthesize beginStageIndex, endStageIndex;
@synthesize dataSource;

- ( id ) initByController: ( AWTableViewController* ) ctrl
{
    self = [ super init ];
    if ( self ) {
        self.controller = ctrl;
        stockedCells = [ [ NSMutableDictionary alloc ] initWithCapacity: 0 ];
        beginStageIndex = -1;
        endStageIndex = -1;
    }
    return self;
}

- ( AWTableCell* ) cellAtIndexPath: ( AWTableIndexPath* ) index
{
    if( !dataSource ) return nil;
 
    int n = index.stage * columnMax + index.column;
    if( [ dataSource respondsToSelector: @selector( numberOfCellsInTableView: ) ] ) {
        int size = [ dataSource numberOfCellsInTableView: self ];
        if( n < size ) {
            if( [ dataSource respondsToSelector: @selector( controller:cellForTableAtIndexPth: ) ] ) {
                AWTableCell* cell = [ dataSource controller: self cellForTableAtIndexPth: index ];
                return cell;
            }
        }
    }
    
    return nil;
}


- ( void ) reloadDataWithStages: ( NSMutableArray* ) containers
{
    // 全セル、ストックセルを削除
    for( int i = 0; i < [ containers count ]; i++ ) {
        AWTableCellContainer* container = [ containers objectAtIndex: i ];
        for( int j = 0; j < [ container.cells count ]; j++ ) {
            AWTableCell* cell = [ container.cells objectAtIndex: j ];
            if( cell ) [ cell removeFromSuperview ];
        }
        [ container removeAllCells ];
    }
    [ stockedCells removeAllObjects ];
    
    for( int i = beginStageIndex; i <= endStageIndex; i++ ) {
        if( i < 0 || i > [ containers count ] ) return;
        AWTableCellContainer* container = [ containers objectAtIndex: i ];
        for( int j = 0; j < columnMax; j++ ) {
            AWTableCell* cell = [ self cellAtIndexPath: [ AWTableIndexPath makeWithStage: i column: j ] ];
            [ container addCell: cell ];
        }
        
    }
    
    
}


#pragma mark - 

- ( int ) rowOfVisibleAtStageIndex: ( int ) iStage
{
    if( iStage >= beginStageIndex && iStage <= endStageIndex ) {
        return iStage - beginStageIndex;
    }
    else {
        return -1;
    }
    
}

- ( void ) containers: ( NSMutableArray* ) containers removeCellAtIndexPath: ( AWTableIndexPath* ) indexPath
{
    int column = indexPath.column;
    int row = [ self rowOfVisibleAtStageIndex: indexPath.stage ];
    if( row < 0 ) return;
    
    AWTableCellContainer* container = [ containers objectAtIndex: row ];
    if( column < 0 || column > [ container.cells count ] ) return;
    
    AWTableCell* cell = [ container removeCellAtIndex: column ];
    [ self addReusableCell: cell ];
}

- ( void ) containers: ( NSMutableArray* ) containers insertCellAtRow: ( int ) row column: ( int ) column cell: ( AWTableCell* ) cell
{
    if( row >= [ containers count ] )  return;
    AWTableCellContainer* container = [ containers objectAtIndex: row ];
    if( column < 0 || column > [ container.cells  count ] ) return;
    
    if( column == 0 ) {
        [ container addCell: cell ];
    }
    else {
        [ container insertCellAtIndex: column cell: cell ];
    }
}

- ( void ) container: ( NSMutableArray* ) containers insertCellAtTail: ( AWTableCell* ) cell
{
    // 最後のステージにセルが入るか判定
    AWTableCellContainer* lastStage = [ containers objectAtIndex: [ containers count ] - 1 ];
    if( [ lastStage.cells count ] < controller.cellCountPerStage ) {
        [ lastStage addCell: cell ];
    }
    else {
        [ self.controller addStage ];
        AWTableCellContainer* container = [ containers objectAtIndex: [ containers count ] - 1 ];
        if( [ container.cells count ] < controller.cellCountPerStage ) {
            [ container addCell: cell ];
        }
    }
}

#pragma mark - cell recycle
- ( void ) addReusableCell: ( AWTableCell* ) cell
{
    NSString *reuseIdentifier = cell.reuseIdentifier;
    if ( !reuseIdentifier ) return;
    
    NSMutableSet* cellSet = [ stockedCells objectForKey: reuseIdentifier ];
    if ( !cellSet ) {
        cellSet = [ [ NSMutableSet alloc ] initWithCapacity: 0 ];
        [ stockedCells setObject: cellSet forKey: reuseIdentifier ];
    }
    //NSLog(@"cellset count: %d", [cellSet count]);
    [ cellSet addObject: cell ];
}

// 
- ( AWTableCell* ) dequeueReusableCellWithIdentifier: ( NSString* ) identifier
{
    NSMutableSet* cellSet = ( NSMutableSet* )[ stockedCells objectForKey: identifier ];
    AWTableCell* cell = [ cellSet anyObject ];
    if ( cell ) [ cellSet removeObject: cell ];
    
    return cell;
}

- ( void ) removeContainer: ( AWTableCellContainer* ) container
{
    for( int i = 0; i < [ container.cells count ]; i++ ) {
        AWTableCell* cell = [ container.cells objectAtIndex: i ];
        [ cell removeFromSuperview ];
        [ self addReusableCell: cell ];
    }
    
    [ container.cells removeAllObjects ];
}


// 表示領域の変更時
- ( void ) layoutSubviewsWithRect: ( CGRect ) rect
{
    float y = rect.origin.y - self.controller.headerView.frame.size.height;
    int top = ( int ) ( y / self.controller.stageSize.height );
    int bottom = top + self.controller.numberOfVisibleStage - 1;
    
    if( top == beginStageIndex ) return;
    
    if( beginStageIndex == -1 ) {
        // セルが何もないとき
        for( int i = top; i < bottom; i++ ) {
            for( int j = 0; j < self.controller.cellCountPerStage; j++ ) {
                AWTableIndexPath* path = [ AWTableIndexPath makeWithStage: i column: j ];
                [ self insertCellAtTail: [ self cellAtIndexPath: path ] ];
            }
        }
        
    }
    else {
        
        if( top < beginStageIndex ) {
            // 上に戻る
            
            // 下のかくれる分を削除
            int delta = beginStageIndex - top;
            int begin = self.controller.cellCountPerStage - delta;
            for( int i = begin; i < begin + delta; i++ ) {
                for( int j = 0; j < self.controller.cellCountPerStage; j++ ) {
                    [ self removeCellAtRow: i column: j ];
                }
            }
            
            // 上のみえてくる分を追加
            for( int i = top; i < top + delta; i++ ) {
                for( int j = 0; j < self.controller.cellCountPerStage; j++ ) {
                    AWTableIndexPath* path = [ AWTableIndexPath makeWithStage: i column: j ];
                    [ self insertCellAtRow: 0 column: 0 cell: [ self cellAtIndexPath: path ] ];
                }
            }
            
        }
        else {
            // 下に
            
            // 上の隠れる分を削除
            int delta = top - beginStageIndex;
            for( int i = 0; i < delta; i++ ) {
                for( int j = 0; j < self.controller.cellCountPerStage; j++ ) {
                    [ self removeCellAtRow: i column: j ];
                }
            }
            
            // 下に出てくる分を追加
            for( int i = top; i < top + delta; i++ ) {
                for( int j = 0; j < self.controller.cellCountPerStage; j++ ) {
                    AWTableIndexPath* path = [ AWTableIndexPath makeWithStage: i column: j ];
                    [ self insertCellAtTail: [ self cellAtIndexPath: path ] ];
                }
            }
        }
        
    }
    
    beginStageIndex = top;
    endStageIndex = bottom;
}



@end
